use async_graphql::dataloader::*;
use async_graphql::*;
use itertools::Itertools;
use std::collections::HashMap;
use std::sync::Arc;

use crate::models::gruppe::Gruppe;
use crate::scalar::Id;

pub struct BenutzerGruppenLoader {
    pub pool: sqlx::PgPool,
}

impl Loader<Id> for BenutzerGruppenLoader {
    type Value = Vec<Gruppe>;
    type Error = Arc<sqlx::Error>;

    async fn load(&self, keys: &[Id]) -> Result<HashMap<Id, Self::Value>, Self::Error> {
        let rows = sqlx::query!(
            r#"
                SELECT
                    bg.benutzer_id, 
                    g.id, 
                    g.gruppenname, 
                    g.erstellt_am, 
                    g.geaendert_am 
                FROM gruppen AS g
                LEFT JOIN benutzer_gruppen AS bg ON g.id = bg.gruppe_id
                WHERE bg.benutzer_id = ANY($1);
            "#,
            keys
        )
        .fetch_all(&self.pool)
        .await?
        .into_iter()
        .map(|row| {
            (
                row.benutzer_id,
                Gruppe {
                    id: row.id,
                    gruppenname: row.gruppenname,
                    erstellt_am: row.erstellt_am,
                    geaendert_am: row.geaendert_am,
                },
            )
        })
        .into_group_map();

        Ok(rows)
    }
}
