use async_graphql::{ComplexObject, Context, FieldResult, SimpleObject};

use crate::{dataloader::LoaderContext, models::gruppe::Gruppe, models::rolle::Rolle, scalar::Id};

use super::gruppe_ansicht::GruppeAnsicht;

#[derive(sqlx::FromRow, SimpleObject)]
#[graphql(complex)]
pub struct Benutzer {
    /// Die UUID eines Benutzers
    pub id: Id,

    /// Die Kennung des Benutzers
    pub kennung: String,

    /// Der Vorname des Benutzers
    pub vorname: String,

    /// Der Nachname des Benutzers
    pub nachname: String,
}

#[ComplexObject]
impl Benutzer {
    /// Die Rollen des Benutzers
    pub async fn rollen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Option<Vec<Rolle>>> {
        let loader = ctx.data::<LoaderContext>()?;
        Ok(loader.benutzer_rollen.load_one(self.id).await?)
    }

    /// Die Gruppen des Benutzers
    pub async fn gruppen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Option<Vec<Gruppe>>> {
        let loader = ctx.data::<LoaderContext>()?;
        Ok(loader.benutzer_gruppen.load_one(self.id).await?)
    }
    /// Die Gruppen eines Benutzer kumulativ mit den Gruppen aus den Rollen
    pub async fn gruppen_kumulativ<'ctx>(
        &self,
        ctx: &Context<'ctx>,
    ) -> FieldResult<Option<Vec<GruppeAnsicht>>> {
        let loader = ctx.data::<LoaderContext>()?;
        Ok(loader.benutzer_gruppen_kumulativ.load_one(self.id).await?)
    }
}
